"""Ingest pipeline for .kgn / .kge files."""

from kgn.ingest.service import IngestBatchResult, IngestFileResult, IngestService

__all__ = [
    "IngestBatchResult",
    "IngestFileResult",
    "IngestService",
]
